@@ -270,10 +270,41 @@ clist_build_children (GladeXML *xml, GtkWidget *w, GladeWidgetInfo *info,
 	gint col = 0;
 
 	for (tmp = info->children; tmp; tmp = tmp->next) {
-		GtkWidget *child = glade_xml_build_widget (xml, tmp->data,
-							   longname);
+		GladeWidgetInfo *cinfo = tmp->data;
+		GtkWidget *child = NULL;
 
-		gtk_clist_set_column_widget (GTK_CLIST(w), col, child);
+		/* if it is a GtkLabel child with a "label" attribute,
+		 * treat it specially. */
+		if (!strcmp(cinfo->class, "GtkLabel")) {
+			GList *tmp2;
+			gchar *label = NULL;
+
+			for (tmp2 = cinfo->attributes; tmp2; tmp2=tmp2->next) {
+				GladeAttribute *attr = tmp2->data;
+
+				if (!strcmp(attr->name, "label")) {
+					label = attr->value;
+					break;
+				}
+			}
+			if (label) {
+				gtk_clist_set_column_title (GTK_CLIST(w), col,
+							    label);
+				/* get the GtkLabel, which is a child
+				 * of the alignment widget used as
+				 * title. */
+				child = gtk_clist_get_column_widget
+					(GTK_CLIST(w), col);
+				child = GTK_BIN(child)->child;
+				glade_xml_set_common_params(xml, child, cinfo,
+							    longname);
+			}
+		}
+		if (!child) {
+			child = glade_xml_build_widget (xml, cinfo, longname);
+
+			gtk_clist_set_column_widget (GTK_CLIST(w), col, child);
+		}
 		col++;
 	}
 }
